@@ -1,34 +1,60 @@
 from __future__ import division
 import operator
-import numpy as np
+import numpy as _np
 import warnings
 from copy import deepcopy
 from collections import OrderedDict
 from .coord import Coords
 
+from ..version import __version__
 
-_numerical_types = (np.ndarray, int, float, complex, np.complex64)
+version = __version__
 
-_nddata_core_version = "1.0"
+
+_numerical_types = (
+    _np.ndarray,
+    int,
+    float,
+    complex,
+    _np.complex64,
+    _np.float16,
+    _np.float32,
+    _np.double,
+    _np.longdouble,
+    _np.csingle,
+    _np.cdouble,
+    _np.clongdouble,
+    _np.longlong,
+)
+
+# _nddata_core_version = "1.0"
 
 
 class ABCData(object):
-    """nddata class"""
+    """N-Dimensional Data Object
+
+    Attributes:
+        values (numpy.ndarray): Data values in
+        dims (list): List of strings giving dimension labels
+        coords (Coords): Collection of numpy.ndarrays defining the axes
+        attrs (dict): dictionary of parameters
+        error (numpy.ndarray): If not None, error for values which are propagated during mathematical operations
+        proc_attrs (list): List of processing steps
+    """
 
     __array_priority__ = 1000  # radd, rsub, ... should return nddata object
 
     def __init__(
-        self, values=np.r_[[]], dims=[], coords=[], attrs={}, error=None, **kwargs
+        self, values=_np.r_[[]], dims=[], coords=[], attrs={}, error=None, **kwargs
     ):
-
-        self._nddata_core_version = _nddata_core_version
+        self.version = version
 
         # if values is list, convert to numpy array
         if isinstance(values, list):
-            values = np.array(values)
+            values = _np.array(values)
 
         # verify values are numpy array
-        if isinstance(values, np.ndarray):
+        if isinstance(values, _np.ndarray):
             self._values = values
         else:
             raise TypeError(
@@ -42,7 +68,7 @@ class ABCData(object):
         else:
             raise TypeError('attrs must be type "dict" not %s' % str(type(attrs)))
 
-        if isinstance(error, np.ndarray) or (error == None):
+        if isinstance(error, _np.ndarray) or (error == None):
             self._error = error
         else:
             raise TypeError(
@@ -92,7 +118,7 @@ class ABCData(object):
         """
 
         for coord in coords:
-            if isinstance(coord, np.ndarray):
+            if isinstance(coord, _np.ndarray):
                 if len(coord.shape) == 1:
                     pass
                 else:
@@ -112,7 +138,7 @@ class ABCData(object):
             bool: True if valid error. False otherwise.
         """
 
-        check_type = isinstance(error, np.ndarray)
+        check_type = isinstance(error, _np.ndarray)
 
         if check_type:
             check_size = error.size == self._values.size
@@ -146,7 +172,7 @@ class ABCData(object):
 
         for key in self._attrs:
             if not isinstance(
-                self._attrs, (list, np.ndarray, int, float, complex, str)
+                self._attrs, (list, _np.ndarray, int, float, complex, str)
             ):
                 return False
 
@@ -181,7 +207,6 @@ class ABCData(object):
 
         # check slices
         for slice_ in index_slice:
-
             # type must be slice or tuple
             if not isinstance(slice_, (slice, tuple, float, int)):
                 raise ValueError("Invalid slice type")
@@ -196,11 +221,11 @@ class ABCData(object):
             if isinstance(slice_, tuple):
                 index = a.index(dim)
                 if len(slice_) == 1:
-                    start = np.argmin(np.abs(slice_[0] - a.get_coord(dim)))
+                    start = _np.argmin(_np.abs(slice_[0] - a.get_coord(dim)))
                     updated_index_slice.append(slice(start, start + 1))
                 else:
-                    start = np.argmin(np.abs(slice_[0] - a.get_coord(dim)))
-                    stop = np.argmin(np.abs(slice_[1] - a.get_coord(dim)))
+                    start = _np.argmin(_np.abs(slice_[0] - a.get_coord(dim)))
+                    stop = _np.argmin(_np.abs(slice_[1] - a.get_coord(dim)))
                     if start == stop:
                         stop = start + 1
                     if stop < start:
@@ -213,7 +238,7 @@ class ABCData(object):
                 else:
                     updated_index_slice.append(slice(slice_, None))
             elif isinstance(slice_, float):
-                start = np.argmin(np.abs(slice_ - a.get_coord(dim)))
+                start = _np.argmin(_np.abs(slice_ - a.get_coord(dim)))
                 updated_index_slice.append(slice(start, start + 1))
             else:
                 updated_index_slice.append(slice_)
@@ -260,7 +285,6 @@ class ABCData(object):
 
         # check slices
         for slice_ in index_slice:
-
             # type must be slice or tuple
             if not isinstance(slice_, (slice, tuple, float, int)):
                 raise ValueError("Invalid slice type")
@@ -275,11 +299,11 @@ class ABCData(object):
             if isinstance(slice_, tuple):
                 index = a.index(dim)
                 if len(slice_) == 1:
-                    start = np.argmin(np.abs(slice_[0] - a.get_coord(dim)))
+                    start = _np.argmin(_np.abs(slice_[0] - a.get_coord(dim)))
                     updated_index_slice.append(slice(start, start + 1))
                 else:
-                    start = np.argmin(np.abs(slice_[0] - a.get_coord(dim)))
-                    stop = np.argmin(np.abs(slice_[1] - a.get_coord(dim)))
+                    start = _np.argmin(_np.abs(slice_[0] - a.get_coord(dim)))
+                    stop = _np.argmin(_np.abs(slice_[1] - a.get_coord(dim)))
                     if start == stop:
                         stop = start + 1
                     if stop < start:
@@ -292,7 +316,7 @@ class ABCData(object):
                 else:
                     updated_index_slice.append(slice(slice_, None))
             elif isinstance(slice_, float):
-                start = np.argmin(np.abs(slice_ - a.get_coord(dim)))
+                start = _np.argmin(_np.abs(slice_ - a.get_coord(dim)))
                 updated_index_slice.append(slice(start, start + 1))
             else:
                 updated_index_slice.append(slice_)
@@ -337,11 +361,12 @@ class ABCData(object):
                 self.attrs[key] = b.attrs[key]
             else:
                 if not self.attrs[key] is b.attrs[key]:
-                    warnings.warn(
-                        "attrs in two dictionarys contain different values, leaving original value:\n{}:{}".format(
-                            key, self.attrs[key]
-                        )
-                    )
+                    pass
+                    # warnings.warn(
+                    #     "attrs in two dictionarys contain different values, leaving original value:\n{}:{}".format(
+                    #         key, self.attrs[key]
+                    #     )
+                    # )
 
     def __len__(self):
         """Returns len(values) the length of the first dimension in values"""
@@ -357,10 +382,19 @@ class ABCData(object):
         sorted_order = sorted(range(len(self.dims)), key=lambda x: self.dims[x])
 
         self.coords.reorder_index(sorted_order)
-        self._values = np.moveaxis(self._values, range(len(sorted_order)), sorted_order)
+        self._values = _np.moveaxis(
+            self._values, range(len(sorted_order)), sorted_order
+        )
 
     def index(self, dim):
-        """Find index of given dimension name"""
+        """Find index of given dimension name
+
+        Args:
+            dim (str): Name of dimension to index
+
+        Returns:
+            int: Index value of dim
+        """
         if dim in self.dims:
             return self.coords.index(dim)
         else:
@@ -368,14 +402,13 @@ class ABCData(object):
 
     def __truediv__(self, b):
         if isinstance(b, ABCData):
-
             a, b = self.align(b)
 
             a.values = a.values / b.values
 
             # error propagation
             if a.error is not None and b.error is not None:
-                error = abs(a.values) * np.sqrt(
+                error = abs(a.values) * _np.sqrt(
                     (self.error / a.values) ** 2.0 + (b.error / a.values) ** 2.0
                 )
             elif b.error is not None:
@@ -406,7 +439,7 @@ class ABCData(object):
 
             # error propagation
             if a.error is not None and b.error is not None:
-                error = abs(a.values) * np.sqrt(
+                error = abs(a.values) * _np.sqrt(
                     (self.error / a.values) ** 2.0 + (b.error / a.values) ** 2.0
                 )
             elif b.error is not None:
@@ -431,7 +464,7 @@ class ABCData(object):
 
             # error propagation
             if a.error is not None and b.error is not None:
-                a.error = np.sqrt(a.error**2.0 + b.error**2.0)
+                a.error = _np.sqrt(a.error**2.0 + b.error**2.0)
             elif b.error is not None:
                 a.error = b.error
 
@@ -454,7 +487,7 @@ class ABCData(object):
 
             # error propagation
             if a.error is not None and b.error is not None:
-                a.error = np.sqrt(a.error**2.0 + b.error**2.0)
+                a.error = _np.sqrt(a.error**2.0 + b.error**2.0)
             elif b.error is not None:
                 a.error = b.error
 
@@ -481,10 +514,10 @@ class ABCData(object):
 
     @values.setter
     def values(self, b):
-        if not isinstance(b, (int, complex, float, np.ndarray)):
+        if not isinstance(b, (int, complex, float, _np.ndarray)):
             raise TypeError('Values must be type "numpy.ndarray" not %s' % type(b))
         if isinstance(b, (int, complex, float)):
-            b = np.array(b)
+            b = _np.array(b)
 
         self._values = b
 
@@ -525,7 +558,7 @@ class ABCData(object):
 
     @error.setter
     def error(self, b):
-        if isinstance(b, (np.ndarray, type(None))):
+        if isinstance(b, (_np.ndarray, type(None))):
             self._error = b
         else:
             raise ValueError('error must be type "numpy.ndarray"')
@@ -548,7 +581,12 @@ class ABCData(object):
             raise ValueError("Dimension name %s is not in dims" % dim)
 
     def reorder(self, dims):
-        """Reorder dimensions"""
+        """Reorder dimensions
+
+        Args:
+            dims (list): List of strings in new order
+
+        """
 
         if not self._check_dims(dims):
             raise TypeError("New dims must be list of str with no duplicates")
@@ -559,12 +597,20 @@ class ABCData(object):
         # Add original dims to end, remove duplicates
         dims = list(OrderedDict.fromkeys(dims + self.dims))
 
+        #        print('original dims', self.dims)
+        #        print('new order of dims', dims)
+
         new_order = [dims.index(dim) for dim in self.dims]
+        permutation_order = [self.dims.index(dim) for dim in dims]
+        #        print('New Order', new_order)
+        #        print('Permutation Order', permutation_order)
 
         self.coords.reorder(dims)
+        #        print(self.coords)
 
         # Transpose values
-        self.values = np.transpose(self.values, new_order)
+        #        self.values = np.transpose(self.values, new_order)
+        self.values = _np.transpose(self.values, permutation_order)
 
     def __str__(self):
         return "values:\n{}\ndims:\n{}\ncoords:\n{}\nattrs:\n{}".format(
@@ -582,17 +628,17 @@ class ABCData(object):
         shape = a.shape
 
         remove_dims = [a.dims[x] for x in range(len(shape)) if shape[x] == 1]
-        values = np.squeeze(a.values)
+        values = _np.squeeze(a.values)
 
         if a.error is not None:
-            a.error = np.squeeze(a.error)
+            a.error = _np.squeeze(a.error)
 
         attrs = a.attrs
 
         for dim in remove_dims:
             out = a.coords.pop(dim)
             if dim not in attrs:
-                attrs[dim] = np.array(out)
+                attrs[dim] = _np.array(out)
             else:
                 warnings.warn("Attribute lost {}:{}".format(dim, out))
 
@@ -648,14 +694,21 @@ class ABCData(object):
 
     @property
     def shape(self):
+        """tuple: Shape of values"""
         return self.values.shape
 
     @property
     def dtype(self):
+        """type: Values type"""
         return self.values.dtype
 
     def sum(self, dim):
-        """Perform sum down given dimension"""
+        """Perform sum down given dimension
+
+        Args:
+            dim (str): Dimension to perform sum down
+
+        """
         a = self.copy()
         index = a.index(dim)
         a.values = a.values.sum(index)
@@ -668,7 +721,7 @@ class ABCData(object):
         """Align two data objects for numerical operations
 
         Args:
-            b: Ojbect to align with self
+            b: Object to align with self
 
         Returns:
             tuple: self and b aligned data objects
@@ -688,7 +741,7 @@ class ABCData(object):
         # re-order
         old_order = list(range(len(new_order)))
         # re-order b values so they match order of all_dims
-        values_b = np.moveaxis(b.values, new_order, old_order)
+        values_b = _np.moveaxis(b.values, new_order, old_order)
         # create new dims where necessary
         values_b = values_b[
             tuple(slice(None) if dim in b.dims else None for dim in all_dims)
@@ -702,7 +755,7 @@ class ABCData(object):
         else:
             error = a.error
         if b.error is not None:
-            error_b = np.moveaxis(b.values, new_order, old_order)
+            error_b = _np.moveaxis(b.values, new_order, old_order)
             error_b = error_b[
                 tuple(slice(None) if dim in b.dims else None for dim in all_dims)
             ]
@@ -712,7 +765,7 @@ class ABCData(object):
         # check coords
         for dim in all_dims:
             if (dim in a.dims) and (dim in b.dims):
-                if not np.allclose(a.get_coord(dim), b.get_coord(dim)):
+                if not _np.allclose(a.get_coord(dim), b.get_coord(dim)):
                     raise ValueError("Coords do not match for dim: %s" % dim)
 
         # merge attrs
@@ -753,7 +806,7 @@ class ABCData(object):
             dim (str): dimension to sort
         """
 
-        sort_array = np.argsort(self.coords[dim])
+        sort_array = _np.argsort(self.coords[dim])
 
         self.coords[dim] = self.coords[dim][sort_array]
 
@@ -767,7 +820,7 @@ class ABCData(object):
         """Split the dimension dim into"""
 
         if isinstance(coord, int):
-            coord = np.arange(coord)
+            coord = _np.arange(coord)
 
         # move dim to end of dims
         dims = self.dims
@@ -795,27 +848,36 @@ class ABCData(object):
         Returns:
             bool: True if sorted, False otherwise.
         """
-        return np.all(self.coords[dim][:-1] <= self.coords[dim][1:])
+        return _np.all(self.coords[dim][:-1] <= self.coords[dim][1:])
 
     @property
     def real(self):
+        """DNPData: DNPData with real part of values"""
         a = self.copy()
-        a.values = np.real(a.values)
+        a.values = _np.real(a.values)
         return a
 
     @property
     def imag(self):
+        """DNPData: DNPData with imaginary part of values"""
         a = self.copy()
-        a.values = np.imag(a.values)
+        a.values = _np.imag(a.values)
         return a
 
     @property
     def abs(self):
+        """DNPData: DNPData with absolute part of values"""
         a = self.copy()
-        a.values = np.abs(a.values)
+        a.values = _np.abs(a.values)
         return a
 
     def concatenate(self, b, dim):
+        """Concatenate DNPData objects
+
+        Args:
+            b (DNPData): Data object to append to current data object
+            dim (str): dimension to concatenate along
+        """
 
         if not dim in b.dims:
             raise ValueError("dim does not exist")
@@ -826,85 +888,122 @@ class ABCData(object):
 
         b.reorder(self.dims)
 
-        self.values = np.concatenate((self.values, b.values), axis=index)
+        self.values = _np.concatenate((self.values, b.values), axis=index)
 
-        self.coords[dim] = np.concatenate(
+        self.coords[dim] = _np.concatenate(
             (
-                np.array(self.coords[dim]).reshape(-1),
-                np.array(b.coords[dim]).reshape(-1),
+                _np.array(self.coords[dim]).reshape(-1),
+                _np.array(b.coords[dim]).reshape(-1),
             )
         )
 
     def new_dim(self, dim, coord):
-        self.coords.append(dim, np.r_[coord])
-        self.values = np.expand_dims(self.values, -1)
+        """Add new dimension with length 1
+
+        Args:
+            dim (str): Name of new dimension
+            coord (int, float): New coord
+        """
+        self.coords.append(dim, _np.r_[coord])
+        self.values = _np.expand_dims(self.values, -1)
 
     def maximum(self, dim):
-        """Return max for given dim"""
+        """Return max for given dim
+
+        Args:
+            dim (str): Dimension to take maximum along
+
+        """
         a = self.copy()
         index = a.dims.index(dim)
 
-        a.values = np.max(a.values, axis=index)
+        a.values = _np.max(a.values, axis=index)
         a.coords.pop(dim)
 
         return a
 
     def argmax(self, dim):
-        """Return argmax for given dim"""
+        """Return value of coord at values maximum for given dim
+
+        Args:
+            dim (str): Dimension to perform operation along
+        """
         a = self.copy()
         index = a.dims.index(dim)
 
-        a.values = a.coords[dim][np.argmax(a.values, axis=index)]
+        a.values = a.coords[dim][_np.argmax(a.values, axis=index)]
         a.coords.pop(dim)
 
         return a
 
     def argmax_index(self, dim):
-        """Return index of argmax for given dim"""
+        """Return index of coord at values maximum for given dim
+
+        Args:
+            dim (str): Dimension to perform operation along
+        """
         a = self.copy()
         index = a.dims.index(dim)
 
-        a.values = np.argmax(a.values, axis=index)
+        a.values = _np.argmax(a.values, axis=index)
         a.coords.pop(dim)
 
         return a
 
     def minimum(self, dim):
-        """Return min for given dim"""
+        """Return min for given dim
+
+        Args:
+            dim (str): Dimension to perform operation along
+        """
         a = self.copy()
         index = a.dims.index(dim)
 
-        a.values = np.min(a.values, axis=index)
+        a.values = _np.min(a.values, axis=index)
         a.coords.pop(dim)
 
         return a
 
     def argmin(self, dim):
-        """Return argmin for given dim"""
+        """Return value of coord at values minimum for given dim
+
+        Args:
+            dim (str): Dimension to perform operation along
+        """
         a = self.copy()
         index = a.dims.index(dim)
 
-        a.values = a.coords[dim][np.argmin(a.values, axis=index)]
+        a.values = a.coords[dim][_np.argmin(a.values, axis=index)]
         a.coords.pop(dim)
 
         return a
 
     def argmin_index(self, dim):
-        """Return index of argmin for given dim"""
+        """Return index of coord at values minimum for given dim
+
+        Args:
+            dim (str): Dimension to perform operation along
+        """
         a = self.copy()
         index = a.dims.index(dim)
 
-        a.values = np.argmin(a.values, axis=index)
+        a.values = _np.argmin(a.values, axis=index)
         a.coords.pop(dim)
 
         return a
 
     @property
     def ndim(self):
+        """str: Number of dimensions"""
         return self.values.ndim
 
     def unfold(self, dim):
-        """Unfold ND data to 2d data"""
+        """Unfold ND data to 2d data
+
+        Args:
+            dim (str): Dimension to make first (length N), all other dimensions unfolded so that values has shape (N x M)
+
+        """
 
         folded_order = self.dims  # Original order of dims
         self.reorder([dim])  # Move dim to first dimension
