@@ -7,10 +7,13 @@ def save(data_object, filename, save_type=None, *args, **kwargs):
     """Save data to h5 format
 
     Args:
-        data_object (dnpdata object) : dnpdata object to save
+        data_object (DNPData) : dnpdata object to save
         filename (str): name of file, must include extension .h5
         save_type (str): Type of file to save (optional). Allowed values: "h5"
 
+    Returns:
+        none
+
     """
 
     if save_type == None:
@@ -33,7 +36,6 @@ def save(data_object, filename, save_type=None, *args, **kwargs):
 
 
 def autodetect(test_name):
-
     if test_name[-1] == os.sep:
         test_name = test_name[:-1]
 
