@@ -1,5 +1,6 @@
-import numpy as np
+import numpy as _np
 import re
+from warnings import warn
 
 from .. import DNPData
 
@@ -89,10 +90,24 @@ _dspfvs_table_13 = {
     96: 2.995,
 }
 
+_required_params = {
+    "acqus": [
+        "SW_h",
+        "RG",
+        "DECIM",
+        "DSPFIRM",
+        "DSPFVS",
+        "BYTORDA",
+        "TD",
+        "SFO1",
+    ],
+    "acqu2s": ["TD", "SW_h"],
+    "acqu3s": ["TD", "SW_h"],
+}
+
 
 def find_group_delay(attrs_dict):
-    """
-    Determine group delay from tables
+    """Determine group delay from tables
 
     Args:
         attrs_dict (dict): dictionary of topspin acquisition parameters
@@ -101,9 +116,13 @@ def find_group_delay(attrs_dict):
         float: Group delay. Number of points FID is shifted by DSP. The ceiling of this number (group delay rounded up) is the number of points should be removed from the start of the FID.
     """
 
+    # This must be revisited
     group_delay = 0
-    if attrs_dict["DSPFIRM"] != 0 and "GRPDLY" in attrs_dict.keys():
+    if attrs_dict["DSPFVS"] >= 13 and "GRPDLY" in attrs_dict.keys():
         group_delay = attrs_dict["GRPDLY"]
+        if group_delay > 0:
+            return group_delay
+
     elif attrs_dict["DECIM"] == 1.0:
         pass
     else:
@@ -123,112 +142,229 @@ def find_group_delay(attrs_dict):
     return group_delay
 
 
-def import_topspin(path, phase_cycle=[0, 90, 180, 270]):
-    """
-    Import topspin data and return dnpdata object
+# This function does too much, should be broken into smaller functions
+def import_topspin(path, assign_vdlist=False, verbose=False):
+    """Import topspin data and return dnpdata object
 
     Args:
         path (str): Directory of data
-        phase_cycle (list): list of phases used for phase cycling (deg, multiples of 90)
+        assign_vdlist: False, or the name of dimension to assign topspin vdlist
+        verbose (bool): Print additional output for troubleshooting
 
     Returns:
         dnpdata: topspin data
     """
-    dir_list = os.listdir(path)
+    dir_list = os.listdir(path)  # All files and folders in directory
+    if verbose:
+        print("Files in directory:")
+        for each in dir_list:
+            print(" ", each)
 
-    if "fid" in dir_list and "ser" not in dir_list:
-        data = load_fid_ser(path, dtype="fid", phase_cycle=phase_cycle)
-    elif "ser" in dir_list:
-        data = load_fid_ser(path, dtype="ser", phase_cycle=phase_cycle)
+    # Load Acquisition Parameters
+    if verbose:
+        print("Loading acqus")
+    acqus_params = load_acqu(os.path.join(path, "acqus"), verbose=verbose)
+
+    dims = [
+        "t2"
+    ]  # this may cause issues if the first dimension is not the direct time dimension
+
+    if "fid" in dir_list:
+        bin_filename = "fid"
     else:
-        raise ValueError("Could Not Identify Data Type in File")
+        bin_filename = "ser"
+
+    if verbose:
+        print("Binary File:", bin_filename)
+
+    if acqus_params["BYTORDA"] == 0:
+        endian = "<"
+    else:
+        endian = ">"
+
+    if verbose:
+        print("endian", endian)
+
+    topspin_major_version = int(acqus_params["topspin"].split(".")[0])
+
+    # Is this incorrect?
+    # Most topspin data I've seen is i4, however, later versions seem to have i8
+    # float is also possible
+    if topspin_major_version >= 4:
+        data_bytes = 8
+    else:
+        data_bytes = 4
+
+    if acqus_params["DTYPA"] == 0:
+        data_type = "i"
+    elif acqus_params["DTYPA"] == 2:
+        data_type = "f"
+
+    data_type = data_type + str(data_bytes)
+
+    if verbose:
+        print("data type:", data_type)
+
+    raw = load_bin(os.path.join(path, bin_filename), dtype=endian + data_type)
+
+    # Is data always complex?
+    values = raw[0::2] + 1j * raw[1::2]  # convert to complex
+
+    group_delay = find_group_delay(acqus_params)
+    if verbose:
+        print("group delay", group_delay)
+
+    group_delay = int(_np.floor(group_delay))  # should this be floor or ceil?
+
+    # why is dividing by 2 required?
+    t2 = 1.0 / acqus_params["SW_h"] * _np.arange(0, int(acqus_params["TD"] / 2))
+    if verbose:
+        print("points in FID:", acqus_params["TD"] / 2)
+
+    # Handle t2 group delay
+    # t2 = t2[slice(group_delay, int(acqus_params["TD"] / 2))] # Alternative method
+    t2 = t2[group_delay:]
+
+    coords = [t2]
+
+    # This will not work for vdlist data
+    if "acqu2s" in dir_list:
+        if verbose:
+            print("Loading acqu2s")
+        acqu2s_params = load_acqu(os.path.join(path, "acqu2s"), verbose=verbose)
+        dims.insert(0, "t1")
+        t1 = 1.0 / acqu2s_params["SW_h"] * _np.arange(0, int(acqu2s_params["TD"]))
+        coords.insert(0, t1)
+
+    # 3d data must be tested
+    if "acqu3s" in dir_list:
+        if verbose:
+            print("Loading acqu3s")
+        acqu3s_params = load_acqu(os.path.join(path, "acqu3s"), verbose=verbose)
+        dims.insert(0, "t3")
+        t3 = 1.0 / acqu3s_params["SW_h"] * _np.arange(0, int(acqu3s_params["TD"]))
+        coords.insert(0, t3)
+
+    new_shape = [len(coords[ix]) if dims[ix] != "t2" else -1 for ix in range(len(dims))]
+    if verbose:
+        print("Raw Data Shape:", _np.shape(values))
+        print("reshaping data to:", new_shape)
+
+    if assign_vdlist:
+        if verbose:
+            print("Assigning vdlist to %s dim" % assign_vdlist)
+        vdlist = topspin_vdlist(path)
+        if assign_vdlist in dims:
+            index = dims.index(assign_vdlist)
+            coords[index] = vdlist
+
+        else:
+            raise ValueError("Could not identify dimension to assign vdlist")
+
+    # reshape values
+    values = values.reshape(new_shape)
+
+    # Handle group delay
+    values = values[..., slice(group_delay, int(acqus_params["TD"] / 2))]
+
+    # create data object
+    topspin_data = DNPData(values, dims, coords, attrs=acqus_params)
+
+    # Add NMR Frequency to attrs
+    topspin_data.attrs["nmr_frequency"] = acqus_params["SFO1"] * 1e6
+
+    # Assign data/spectrum type
+    topspin_data.attrs["experiment_type"] = "nmr_spectrum"
+
+    # reorder so that 't2' is first
+    topspin_data.reorder(["t2"])
+
+    return topspin_data
+
+
+# Load topspin should also handle this
+def load_pdata(path, verbose=False):
+    """Import prospa processed data
+
+    Args:
+        path (str): Directory of pdata
+        verbose (bool): If true, print output for troubleshooting
+
+    Returns:
+        DNPData: Topspin processed data
+    """
+
+    # Directory
+    dir_list = os.listdir(path)  # All files and folders in directory
+    if verbose:
+        print("Files in directory:")
+        for each in dir_list:
+            print(" ", each)
+
+    proc_params = load_acqu(os.path.join(path, "procs"), verbose=verbose)
+
+    if proc_params["BYTORDP"] == 0:
+        endian = "<"
+    else:
+        endian = ">"
+
+    if verbose:
+        print("endian", endian)
+
+    real_raw = load_bin(os.path.join(path, "1r"), dtype=endian + "i4")
+    imag_raw = load_bin(os.path.join(path, "1i"), dtype=endian + "i4")
+
+    SW = proc_params["SW_p"]
+    offset = proc_params["OFFSET"]  # Reference Offset in ppm
+    td_eff = proc_params["TDeff"]
+    SI = proc_params["SI"]  # What does SI stand for?
+    spectrometer_frequency = proc_params["SF"]  # spectrometer frequency in MHz
+    phase_0 = proc_params["PHC0"]  # Phase correction, zeroth order phase
+    phase_1 = proc_params["PHC1"]  # Phase correction, first order phase
+
+    f2 = (
+        -1 * SW * _np.linspace(0, 1, num=SI, endpoint=False) / spectrometer_frequency
+        + offset
+    )
+
+    raw = real_raw + 1j * imag_raw
+
+    data = DNPData(raw, ["f2"], [f2], attrs=proc_params)
+    data.attrs["nmr_frequency"] = spectrometer_frequency
+    data.attrs["phase_0"] = phase_0
+    data.attrs["phase_1"] = phase_1
 
     return data
 
 
-def load_acqu_proc(path="1", param_filename="acqus", proc_num=1):
-    """
-    Import topspin acqu or proc files
+# Is this function obsolete?
+def load_acqu(path, required_params=None, verbose=False):
+    """Import topspin acqu or proc files
 
     Args:
         path (str): directory of acqu or proc file
-        param_filename (str): parameters filename
-        proc_num (int): number of the folder inside the pdata folder
+        required_params (list): Only return parameters given
+        verbose (bool): If true, print output for troubleshooting
 
     Returns:
         dict: Dictionary of acqusition parameters
     """
 
-    if "proc" in param_filename:
-        path_filename = os.path.join(path, "pdata", str(proc_num), param_filename)
+    raw_params = load_topspin_jcamp_dx(path, verbose=False)
+
+    if required_params is not None:
+        acqus_params = {}
+        for key in required_params:
+            acqus_params[key] = raw_params[key]
     else:
-        path_filename = os.path.join(path, param_filename)
+        acqus_params = raw_params
 
-    # Import parameters
-    with open(path_filename, "r") as f:
-        raw_params = f.read()
-
-    # Split parameters by line
-    lines = raw_params.strip("\n").split("\n")
-    attrs_dict = {}
-
-    # Parse Parameters
-    for line in lines:
-        if line[0:3] == "##$":
-            line_split = line[3:].split("= ")
-            try:
-                attrs_dict[line_split[0]] = float(line_split[1])
-            except:
-                attrs_dict[line_split[0]] = line_split[1]
-            # if line_split[0] in ["TD", "NS"]:
-            # print(line_split[0] + ": " + str(attrs_dict[line_split[0]]))
-
-    needed_params = [
-        "SW_h",
-        "RG",
-        "DECIM",
-        "DSPFIRM",
-        "DSPFVS",
-        "BYTORDA",
-        "TD",
-        "SFO1",
-    ]
-    needed_params_2 = ["SW_h", "TD", "SFO1"]
-    if param_filename in ["acqu", "acqus"]:
-        if not all(
-            map(
-                attrs_dict.keys().__contains__,
-                needed_params,
-            )
-        ):
-            raise KeyError(
-                "Unable to find all needed fields in the " + param_filename + " file"
-            )
-        else:
-            attrs_dict = {x: attrs_dict[x] for x in needed_params}
-
-    elif param_filename in ["acqu2", "acqu2s"]:
-        if not all(map(attrs_dict.keys().__contains__, needed_params_2)):
-            raise KeyError(
-                "Unable to find all needed fields in the " + param_filename + " file"
-            )
-        else:
-            attrs_dict = {x + "_2": attrs_dict[x] for x in needed_params_2}
-
-    elif param_filename in ["acqu3", "acqu3s"]:
-        if not all(map(attrs_dict.keys().__contains__, needed_params_2)):
-            raise KeyError(
-                "Unable to find all needed fields in the " + param_filename + " file"
-            )
-        else:
-            attrs_dict = {x + "_3": attrs_dict[x] for x in needed_params_2}
-
-    return attrs_dict
+    return acqus_params
 
 
+# Legacy
 def load_fid_ser(path, dtype="fid", phase_cycle=None):
-    """
-    Import topspin fid or ser file
+    """This function is deprecated and will be removed from future releases. Please use import_topspin instead. Import topspin fid or ser file
 
     Args:
         path (str): Directory of data
@@ -238,6 +374,13 @@ def load_fid_ser(path, dtype="fid", phase_cycle=None):
     Returns:
         dnpdata: Topspin data
     """
+
+    warn(
+        "This function is deprecated. Please use load_topspin instead. load_fid_ser will be removed after 01/01/2023",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+
     if isinstance(phase_cycle, list):
         for indx, x in enumerate(phase_cycle):
             if x in [0, 360, 720, 1080]:
@@ -252,13 +395,13 @@ def load_fid_ser(path, dtype="fid", phase_cycle=None):
     dir_list = os.listdir(path)
 
     attrs_dict_list = [
-        load_acqu_proc(path, x) for x in ["acqus", "acqu2s", "acqu3s"] if x in dir_list
+        load_acqu(path, x) for x in ["acqus", "acqu2s", "acqu3s"] if x in dir_list
     ]
     attrs_dict = {}
     for a in attrs_dict_list:
         attrs_dict.update(a)
 
-    important_params_dict = {
+    attrs = {
         "nmr_frequency": attrs_dict["SFO1"] * 1e6,
         "SW_h": attrs_dict["SW_h"],
         "TD": attrs_dict["TD"],
@@ -269,7 +412,7 @@ def load_fid_ser(path, dtype="fid", phase_cycle=None):
         for x in ["SW_h_2", "TD_2", "SFO1_2", "SW_h_3", "TD_3", "SFO1_3"]
         if x in attrs_dict.keys()
     }
-    important_params_dict.update(higher_dim_pars)
+    attrs.update(higher_dim_pars)
 
     if attrs_dict["BYTORDA"] == 0:
         endian = "<"
@@ -277,19 +420,23 @@ def load_fid_ser(path, dtype="fid", phase_cycle=None):
         endian = ">"
 
     if dtype == "fid":
-        raw = np.fromfile(os.path.join(path, "fid"), dtype=endian + "i4")
+        raw = _np.fromfile(os.path.join(path, "fid"), dtype=endian + "i4")
     else:
-        raw = np.fromfile(os.path.join(path, "ser"), dtype=endian + "i4")
+        raw = _np.fromfile(os.path.join(path, "ser"), dtype=endian + "i4")
 
     data = raw[0::2] + 1j * raw[1::2]  # convert to complex
 
     group_delay = find_group_delay(attrs_dict)
-    group_delay = int(np.ceil(group_delay))
+    group_delay = int(_np.ceil(group_delay))
 
-    t = 1.0 / attrs_dict["SW_h"] * np.arange(0, int(attrs_dict["TD"] / 2) - group_delay)
+    t = (
+        1.0
+        / attrs_dict["SW_h"]
+        * _np.arange(0, int(attrs_dict["TD"] / 2) - group_delay)
+    )
 
     if "vdlist" in dir_list:
-        important_params_dict.update({"VDLIST": topspin_vdlist(path)})
+        attrs.update({"VDLIST": topspin_vdlist(path)})
 
     if dtype == "fid":
         data = data[group_delay : int(attrs_dict["TD"] / 2)] / attrs_dict["RG"]
@@ -303,16 +450,16 @@ def load_fid_ser(path, dtype="fid", phase_cycle=None):
         coords = [t, range(int(attrs_dict["TD_2"]))]
         dims = ["t2", "t1"]
         if "acqu2s" in dir_list and "acqu3s" not in dir_list:
-            if "VDLIST" in important_params_dict.keys():
-                if len(important_params_dict["VDLIST"]) == int(attrs_dict["TD_2"]):
-                    coords = [t, important_params_dict["VDLIST"]]
+            if "VDLIST" in attrs.keys():
+                if len(attrs["VDLIST"]) == int(attrs_dict["TD_2"]):
+                    coords = [t, attrs["VDLIST"]]
             else:
                 if isinstance(phase_cycle, list):
-                    length1d = int((np.ceil(attrs_dict["TD"] / 256.0) * 256) / 2)
+                    length1d = int((_np.ceil(attrs_dict["TD"] / 256.0) * 256) / 2)
                     ser_data = data.reshape(-1, int(length1d)).T
                     ser_data = ser_data[group_delay : int(attrs_dict["TD"] / 2), :]
-                    phs_facs = np.tile(
-                        np.array(phase_cycle),
+                    phs_facs = _np.tile(
+                        _np.array(phase_cycle),
                         int(attrs_dict["TD_2"]) / len(phase_cycle),
                     )
                     for indx in range(int(attrs_dict["TD_2"])):
@@ -325,16 +472,16 @@ def load_fid_ser(path, dtype="fid", phase_cycle=None):
             t = (
                 1.0
                 / attrs_dict["SW_h"]
-                * np.arange(
+                * _np.arange(
                     0, int(attrs_dict["TD"] / 2 / int(attrs_dict["TD_3"])) - group_delay
                 )
             )
-            if "VDLIST" in important_params_dict.keys() and len(
-                important_params_dict["VDLIST"]
-            ) == int(attrs_dict["TD_2"]):
+            if "VDLIST" in attrs.keys() and len(attrs["VDLIST"]) == int(
+                attrs_dict["TD_2"]
+            ):
                 coords = [
                     t,
-                    important_params_dict["VDLIST"],
+                    attrs["VDLIST"],
                     range(int(attrs_dict["TD_3"])),
                 ]
             else:
@@ -351,7 +498,7 @@ def load_fid_ser(path, dtype="fid", phase_cycle=None):
             )
         data = ser_data
 
-    return DNPData(data, dims, coords, important_params_dict)
+    return DNPData(data, dims, coords, attrs)
 
 
 def topspin_vdlist(path):
@@ -386,11 +533,33 @@ def topspin_vdlist(path):
             value = float(line)
             vdlist.append(value)
 
-    vdlist = np.array(vdlist)
+    vdlist = _np.array(vdlist)
     return vdlist
 
 
+# Legacy import ser
 def load_ser(path, dtype=">i4"):
+    """Depreciated. Use load bin. Import Topspin Ser file
+
+    Args:
+        path (str): Directory of data
+        dtype (str): data format for import
+
+    returns:
+        raw (np.ndarray): Data from ser file
+    """
+    warn(
+        "This function is deprecated, use load_bin instead",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+
+    raw = _np.fromfile(os.path.join(path), dtype=dtype)
+
+    return raw
+
+
+def load_bin(path, dtype=">i4"):
     """Import Topspin Ser file
 
     Args:
@@ -401,14 +570,13 @@ def load_ser(path, dtype=">i4"):
         raw (np.ndarray): Data from ser file
     """
 
-    raw = np.fromfile(os.path.join(path), dtype=dtype)
+    raw = _np.fromfile(os.path.join(path), dtype=dtype)
 
     return raw
 
 
 def load_title(path="1", title_path=os.path.join("pdata", "1"), title_filename="title"):
-    """
-    Import Topspin Experiment Title File
+    """Import Topspin Experiment Title File
 
     Args:
         path (str): Directory of title
@@ -428,21 +596,23 @@ def load_title(path="1", title_path=os.path.join("pdata", "1"), title_filename="
     return title
 
 
-def topspin_jcamp_dx(path):
-    """
-    Return the contents of topspin JCAMP-DX file as dictionary
+def load_topspin_jcamp_dx(path, verbose=False):
+    """Return the contents of topspin JCAMP-DX file as dictionary
 
     Args:
-        path: Path to file
+        path (str): Path to file
+        verbose (bool): If true, print output for troubleshooting
 
     Returns:
-        dict: Dictionary of JCAMP-DX file
+        dict: Dictionary of JCAMP-DX file parameters
     """
 
     attrs = {}
 
     with open(path, "r") as f:
         for line in f:
+            if verbose:
+                print(line)
             line = line.rstrip()
 
             if line[0:3] == "##$":
@@ -461,22 +631,30 @@ def topspin_jcamp_dx(path):
                     array = []
                     if same_line_array != "":
                         same_line_array = same_line_array.split(" ")
-                        same_line_array = [
-                            float(x) if "." in x else int(x) for x in same_line_array
-                        ]
+
+                        try:
+                            same_line_array = [
+                                float(x) if "." in x else int(x)
+                                for x in same_line_array
+                            ]
+                        except:
+                            pass  # Needed in case where "<>" found in some arrays
 
                         array += same_line_array
 
                     while len(array) < array_size:
                         array_line = f.readline().rstrip().split(" ")
 
-                        array_line = [
-                            float(x) if "." in x else int(x) for x in array_line
-                        ]
+                        try:
+                            array_line = [
+                                float(x) if "." in x else int(x) for x in array_line
+                            ]
+                        except:
+                            pass  # Needed in case where "<>" found in some arrays
 
                         array += array_line
 
-                    array = np.array(array)
+                    array = _np.array(array)
 
                     attrs[key] = array
 
@@ -509,6 +687,11 @@ def topspin_jcamp_dx(path):
                     attrs[key] = value
 
             elif line[0:2] == "##":
+                # Extract Title and TopSpin Version, needed for data type determination
+                #                if 'TOPSPIN' in line.upper():
+                if "TITLE" in line:
+                    version = line.split(" ")[-1]
+                    attrs["topspin"] = version
                 try:
                     key, value = tuple(line[2:].split("= ", 1))
                 except:
